/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-10-22 15:28:49.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.service.impl;

import cn.com.qmth.examcloud.commons.exception.StatusException;
import cn.com.qmth.examcloud.core.print.common.jpa.SearchBuilder;
import cn.com.qmth.examcloud.core.print.common.jpa.SpecUtils;
import cn.com.qmth.examcloud.core.print.common.upyun.UpYunClient;
import cn.com.qmth.examcloud.core.print.common.utils.Check;
import cn.com.qmth.examcloud.core.print.entity.PrintingTemplate;
import cn.com.qmth.examcloud.core.print.enums.TemplateType;
import cn.com.qmth.examcloud.core.print.repository.PrintingTemplateRepository;
import cn.com.qmth.examcloud.core.print.service.PrintingTemplateService;
import cn.com.qmth.examcloud.core.print.service.bean.printingtemplate.PrintingTemplateConvert;
import cn.com.qmth.examcloud.core.print.service.bean.printingtemplate.PrintingTemplateInfo;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

import static cn.com.qmth.examcloud.core.print.common.Constants.SYS_CODE_500;

/**
 * @author: fengdesheng
 * @since: 2018/10/30
 */
@Service
public class PrintingTemplateServiceImpl implements PrintingTemplateService {
    private static final Logger log = LoggerFactory.getLogger(PrintingTemplateServiceImpl.class);
    @Autowired
    private PrintingTemplateRepository printingTemplateRepository;
    @Autowired
    private UpYunClient upYunClient;

    @Override
    public List<PrintingTemplateInfo> getPrintingTemplateList(Long orgId, Long examId) {
        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", orgId)
                .eq("examId", examId);
        Specification<PrintingTemplate> spec = SpecUtils.buildSearchers(PrintingTemplate.class, searches.build());
        List<PrintingTemplate> list = printingTemplateRepository.findAll(spec);
        return PrintingTemplateConvert.fillMoreTypes(list);
    }

    @Override
    public PrintingTemplateInfo getPrintingTemplateById(Long id) {
        Optional<PrintingTemplate> optional = printingTemplateRepository.findById(id);
        if (!optional.isPresent()) {
            throw new StatusException(SYS_CODE_500, "当前模板信息不存在！");
        }
        return PrintingTemplateConvert.of(optional.get());
    }

    @Override
    public void savePrintingTemplate(PrintingTemplateInfo info) {
        Check.isNull(info, "模板信息不能为空！");
        Check.isNull(info.getOrgId(), "学校ID不能为空！");
        Check.isNull(info.getExamId(), "考试ID不能为空！");
        Check.isNull(info.getTemplateType(), "模板类型不能为空！");
        Check.isFalse(TemplateType.isExist(info.getTemplateType()), "模板类型不正确！");
        Check.isNull(info.getFileUrl(), "模板文件地址不能为空！");

        SearchBuilder searches = new SearchBuilder()
                .eq("orgId", info.getOrgId())
                .eq("examId", info.getExamId())
                .eq("templateType", info.getTemplateType());
        Specification<PrintingTemplate> spec = SpecUtils.buildSearchers(PrintingTemplate.class, searches.build());

        //截取文件名
        info.setFileName(info.getFileUrl().replace(upYunClient.getUrlPrefix(), ""));

        //某学校考试下一种模板类型只有一条记录
        Optional<PrintingTemplate> optional = printingTemplateRepository.findOne(spec);
        if (optional.isPresent()) {
            //存在记录时，则修改
            PrintingTemplate template = optional.get();
            template.setFileName(info.getFileName());
            template.setFileUrl(info.getFileUrl());
            printingTemplateRepository.save(template);
            return;
        }

        //否则，新增记录
        printingTemplateRepository.save(PrintingTemplateConvert.of(info));
    }

}